function ris=newton_arneodo_orbit(V,aold,bold,power)

% Newton-scheme to find periodic orbit of Arneodo system

F_V=arneodo_orbit_map_fourier(V,aold,bold);
norm(F_V,inf)
while norm(F_V,inf)>10^(-power)
   J1=jacobian_arneodo_orbit_fourier(V,aold,bold);
    V1=V-J1\F_V;
    V=V1;
    F_V=arneodo_orbit_map_fourier(V,aold,bold);
    norm(F_V,inf)
end
ris=V;

end

function [Fc] = arneodo_orbit_map_fourier(V, aold,bold)

%system for the periodic solution of Arneodo system in Fourier 


%%%%%% NB:  the first element of vector is the frequency L=2*pi/period %%%
% input: vector=[L,a1_0,a2_0,a3_0,a1_k,b1_k,a2_k,a2_k,a3_k,b3_k], k=1:m-1
% output F=(phase condition,
%           f^1_01,f^2_01,f^3_01,f^1_k1,f^1_k2,f^2_k1,f^2_k2,f^3_k1,f^3_k2)
% f^j_k1 real part f^j_k2 imaginary part of the j-th equation of Lorenz in
% terms of fourier coefficient 

global alpha beta 

% initial data arrangment
X0=V(1:4);  %[P,a1_0,a2_0,a3_0]
X=V(5:end);

m=length(X)/6+1;  %vector is in R(6m-2)
X=reshape(X,6,m-1); %X=[a1_1,....a1_(m-1)
                  %     b1_1,....b1_(m-1)
                  %     a2_1,....a2_(m-1)
                  %       .        .
                  %     b3_1,....b3_(m-1)]
                  

%aold=[X0(2:end),[X(1,:);X(3,:);X(5,:)]];    % aj_k old solution
%bold=[ [0;0;0], [X(2,:);X(4,:);X(6,:)]];    % bj_k old solution

%L=2*pi/P;

L=X0(1);

a01=X0(2);
a02=X0(3);
a03=X0(4);

C1=[a01,(X(1,:)+1i*X(2,:))];     % complex vectors
C2=[a02,(X(3,:)+1i*X(4,:))];
C3=[a03,(X(5,:)+1i*X(6,:))];

% convolution: conv12(k)= sum(j+l=k)C1_j*C2_l
conv11=complex_convolution(C1,C1);
conv11(1)=real(conv11(1));
% evaluation phase condition <u(0)-u_old(0),dot(u_old)(0)>

Diff0=[a01;a02;a03]-aold(:,1)+2*[sum(X(1,:)-aold(1,2:end));sum(X(3,:)-aold(2,2:end));sum(X(5,:)-aold(3,2:end))];
Velold0=zeros(3,1);
for k=1:m-1
    Velold0=Velold0+k*bold(:,k+1);
end

phase=Diff0'*Velold0;

% evaluation F
Fc=zeros(4+6*(m-1),1);

f01=-a02;
f02=-a03;
f03=-alpha*a01+conv11(1)+beta*a02+a03;

Fc(1:4)=[phase;f01;f02;f03];

for k=1:m-1
    f1k=L*k*1i*C1(k+1)-C2(k+1);
    f2k=L*k*1i*C2(k+1)-C3(k+1);
    f3k=L*k*1i*C3(k+1)-alpha*C1(k+1)+conv11(k+1)+beta*C2(k+1)+C3(k+1);
    Fc(4+(k-1)*6+1:4+(k-1)*6+6)=[real(f1k);imag(f1k);real(f2k);imag(f2k);real(f3k);imag(f3k)];
end
end

function conv2=complex_convolution(A,B)
 
 % given A=(a0,a1,a2,....aN), B=(b0,b1,b2...bN)
 % first construct the conjugate estension (a_-N...a_-2,a_-1,a0,a1,...aN)
 % and (b_-N...b_-2,b_1,b0,b1,..bN) with a,b_(-i)=conj(a,b_i)
 % then compute conv(A,B)_k= sum_(j+l=k) aj*bl   k in [0,2*N] 
 
S=size(A);
if S(1)>1
    A=transpose(A);
    B=transpose(B);
end
  m=length(A);
  CA=conj(A(2:end));
  CB=conj(B(2:end));
  Atot=[fliplr(A),CA];
  Btot=[fliplr(CB),B];
  row=Atot;
  Matr=Atot;
  if isa(A,'intval')==1
      i0=intval(0);
  else
      i0=0;
  end
  for i=1:m-1
      row=circshift(row,[0,1]);
      row(1)=i0;
      Matr=[Matr;row];
  end
  Matr=[Matr;zeros(m-1,m),Matr(2:m,2:m)];
  conv2=Matr*Btot.';
  conv2=conv2.';
end