function [B,Qk]=integration_floquet_KS_complex(mu_k)
global nu T 

N=size(mu_k,1);
m=(size(mu_k,2)-1)/2;
   
      
[M,~,~]=KS_monodromyT(mu_k); 

% first guess for the monodromy matrix
B=logm(M)/(T);

% first guess for the coefficients Qk ( very rough, they will be optimized
% in the Newton scheme

Qk=zeros(N,N,2*m+1);
Qk(:,:,m+1)=eye(N);
return

function [Mon,t1,y1]=KS_monodromyT(mu_k)
                   
        options=odeset('AbsTol',1e-13,'RelTol',1e-11);
        time=0;
        sol_ini=mu_k*exp(1i*[-m:1:m]*2*pi/T*time).';
        Id=eye(N);         
        ini1=[real(sol_ini);reshape(Id,[],1)];        
        [t1,y1]=ode45(@systemMonodromyreal,linspace(0,T,1000),ini1,options);       
        Mon=reshape(y1(end,N+1:end),N,N);                     % monodromy matrix
      
    function df=systemMonodromyreal(~,x)
        
            gamma=x(1:N);
            Mon=reshape(x(N+1:end),N,N);  
            Q=quadratic_sumFFToo(gamma,gamma);
            df(1:N,1)=([1:1:N].^2-nu*[1:1:N].^4).'.*gamma-[1:1:N].'.*Q;
            
            Mat1=zeros(N);
            Mat2=zeros(N);
            gammaext=[gamma;zeros(2*N,1)];
            for k=1:N
                Mat1(k+1:end,k)=gamma(1:N-k);
                Mat2(1:end,k)=gammaext(k+1:k+N);
            end
            Mat=Mat1-Mat1'-Mat2;
 
            J=diag([1:1:N].^2-nu*[1:1:N].^4)-2*repmat([1:1:N].',1,N).*Mat;
            dfM=J*Mon;
            df=[df;reshape(dfM,[],1)];
%                 
        end
        
        
    end

 end
function [s]=quadratic_sumFFToo(a1,a2)

a1=[0;a1];
a2=[0;a2];
m=length(a1);

b1=flipdim(a1(2:m),1);
ta1=[zeros(m,1);-b1;a1;zeros(m,1)];
tu1=ifft(ifftshift(ta1));

b2=flipdim(a2(2:m),1);
ta2=[zeros(m,1);-b2;a2;zeros(m,1)];
tu2=ifft(ifftshift(ta2));

F=fftshift(fft(tu1.*tu2));

s=real((4*m-1)*F(2*m+1:3*m-1));
end