
%% computation of the numerical solution 
%% for the periodic orbit of the Lorenz system,
%% Then solve the Floquet system for P_k and B for T-periodic complex P(t)

global rho beta sigma 
beta=8/3;sigma=10;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     %%%% USER-DEFINED VARIABLES %%%%%%%
rho=input('Please choose the parameter rho, e.g. 28 or 100.55 \n');    

if rho==28
       Mgamma=66;                    % number of Fourier Coefficients of the periodic solution
       m=66;                         % number of Fourier Coefficients of P(t) 
       mode = 'T'
       dim_man=1
       'SCALE EINGENVALUES = NO'
       scaleEigs=1;
elseif rho==100.55
    Mgamma=90;                    % number of Fourier Coefficients of the periodic solution
    m=115;                         % number of Fourier Coefficients of Q(t) 
    mode='2T'; %2T%
    dim_man=2;
    'SCALE EINGENVALUES = yes'
    'PLEASE SET STABILITY = S'
    scaleEigs = [0.0025, 0.075];
else
    Mgamma=50;
    m=66;                         % number of Fourier Coefficients of P(t) 
    mode=input('Select the Floquet normal form: (T) for Complex or (2T) for Real \n','s');
    dim_man=input('Dimension of manifold ( out of the time dimension) \n');
    'scaling = off'
    scaleEigs=ones(1,dim_man);
end
    
%Manifold Parameters
stability=input('Please choose the stability of the manifold: S stable, U unstable ','s');
%stability='S'; % 'S' or 'U'
N_manifold=10;
m_manifold=m;
 
 D2=[0,0,0;-1,1,3;1,1,2]; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% COMPUTATION OF NUMERICAL PERIODIC ORBIT %%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Computing numerical periodic solution for the Lorenz system');

load LorNumericalSolutions.mat
[diffrho,NumSol]=min(abs(LorNumericalSolutions(1,:)-rho));

if abs(diffrho)>0.01
    disp('doing continuation....');
    Sol_ini=LorNumericalSolutions(2:end,NumSol);
    if length(Sol_ini)<6*(Mgamma+1)-2
        Sol_ini=[Sol_ini;zeros(6*(Mgamma+1)-2-length(Sol_ini),1)];
    end
    NumericalSol=continuation_lorenz(LorNumericalSolutions(1,NumSol),Sol_ini,rho);
else
    NumericalSol=LorNumericalSolutions(2:end,NumSol);
end

V=NumericalSol(1:6*(Mgamma+1)-2,1);


X0=V(1:4);                                     %[L,a1_0,a2_0,a3_0]
X=V(5:end); 
X=reshape(X,6,[]);
aold=[X0(2:end),[X(1,:);X(3,:);X(5,:)]];
bold=[ [0;0;0], [X(2,:);X(4,:);X(6,:)]];


power=13;                           
orbit=newton_orbit_lorenz(V,aold,bold,power);                                % refine the numerical solution of the periodic orbit
L=orbit(1);     % frequence of the orbit

% if wish to save the orbit into the storage LorNumericalSolutions
%     x=[rho;orbit];
%     if length(x)>359
%         x(360:end)=[];
%     else
%         x=[x;zeros(359-size(x,1),1)];
%     end
%     LorNumericalSolutions=[x,LorNumericalSolutions];
%      save LorNumericalSolutions LorNumericalSolutions;

clear LorNumericalSolutions

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% COMPUTE NUMERICAL SOLUTION FOR FLOQUET SYSTEM %%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

global  gamma_0 gamma_k T

        
T=2*pi/L;                                                                % period of the orbit
gamma_0=orbit(2:4);                                                               % a0 coefficients
ABk=reshape(orbit(5:end),6,[]); 

% each column of ABk contains the real and 
%immaginary parts of the coefficients:
%[Re(xi^1_k);Im(xi^1_k);Re(xi^2_k);Im(xi^2_k);Re(xi^3_k);Im(xi^3_k))
%ak,bk coefficients of the orbit

gamma_k=ABk([1,3,5],:)+1i*ABk([2,4,6],:);
orbit=[flipdim(conj(gamma_k(:,1:end)),2),gamma_0,gamma_k];
disp('Computating the numerical solution for Floquet system');
  if mode=='2T'
  T=2*T;
  gamma_k2T=zeros(3,2*size(gamma_k,2));
  gamma_k2T(:,2:2:end)=gamma_k;
  gamma_k=gamma_k2T;
  end
  
Ak=compute_Ak_Lorenz(gamma_0,gamma_k);

[B,Pk]=integration_floquet_lorenz_complex(m);

return



















