function orbit=newton_orbit_lorenz(V,aold,bold,power)

%Newton scheme to determine the numerical solution of the Lorenz system

F_V=map_orbit_lorenz(V,aold,bold);
counter=0;
while norm(F_V,inf)>10^(-power)
    J1=jacobian_orbit_lorenz(V,bold);
    V1=V-J1\F_V;
    V=V1;
    F_V=map_orbit_lorenz(V,aold,bold);
    display(['InfNorm=' num2str(norm(F_V,inf))]);
    counter=counter+1;
    if counter==10
        power=power-1;
    end
    if counter==12
        power=power-1;
    end
    if counter>15
        display(['Newton scheme does not converge']);
        orbit=[];
        return
    end
end
orbit=V;




end

function [Fc] =map_orbit_lorenz(V,aold,bold)


% input: vector=[L,a1_0,a2_0,a3_0,a1_k,b1_k,a2_k,a2_k,a3_k,b3_k], k=1:m-1
% output F=(phase condition,
%           f^1_01,f^2_01,f^3_01,f^1_k1,f^1_k2,f^2_k1,f^2_k2,f^3_k1,f^3_k2)
% f^j_k1 real part f^j_k2 imaginary part of the j-th equation of Lorenz in terms of fourier coefficient 

% for non-interval entries


global rho beta sigma

% initial data arrangment
X0=V(1:4);  
X=V(5:end);

m=length(X)/6+1;  
X=reshape(X,6,m-1); %  X=[a1_1,....a1_(m-1)      Real(Xi_1)
                    %     b1_1,....b1_(m-1)      Imag(Xi_1)
                    %     a2_1,....a2_(m-1)      Real(Xi_2)
                    %       .        .                .
                    %     b3_1,....b3_(m-1)]     Imag(X1_3)
                 

L=X0(1);

a01=X0(2);
a02=X0(3);
a03=X0(4);

C1=[a01,(X(1,:)+1i*X(2,:))];     
C2=[a02,(X(3,:)+1i*X(4,:))];
C3=[a03,(X(5,:)+1i*X(6,:))];

% convolution: conv12(k+1)= sum(j+l=k)C1_j*C2_l   k=0,1... 
conv12=complex_convolution(C1,C2);
conv13=complex_convolution(C1,C3);

% evaluation phase condition <u(0)-u_old(0),dot(u_old)(0)>
Diff0=[a01;a02;a03]-aold(:,1)+2*[sum(X(1,:)-aold(1,2:end));sum(X(3,:)-aold(2,2:end));sum(X(5,:)-aold(3,2:end))];
Velold0=zeros(3,1);
for k=1:m-1
    Velold0=Velold0+k*bold(:,k+1);
end

phase=Diff0'*Velold0;

% evaluation F
Fc=zeros(4+6*(m-1),1);

f01=-sigma*(a02-a01);
f02=-(rho*a01-a02-real(conv13(1)));
f03=-(-beta*a03+real(conv12(1)));
Fc(1:4)=[phase;f01;f02;f03];

for k=1:m-1
    f1k=L*k*1i*C1(k+1)-sigma*(C2(k+1)-C1(k+1));
    f2k=L*k*1i*C2(k+1)-(rho*C1(k+1)-C2(k+1)-conv13(k+1));
    f3k=L*k*1i*C3(k+1)-(-beta*C3(k+1)+conv12(k+1));
    Fc(4+(k-1)*6+1:4+(k-1)*6+6)=[real(f1k);imag(f1k);real(f2k);imag(f2k);real(f3k);imag(f3k)];
end

end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function jacob=jacobian_orbit_lorenz(V,bold)


%given V\in R(6m-2) and the a_k, b_k coefficient of the old solution ( for
%the phase condition
%returns the Jacobian matrix jacob\in M(6m-2x6m-2) of the Lorenz system in
%fourier coefficient evaluated in V

global rho beta sigma

m=(length(V)+2)/6;
L=V(1);
a01=V(2);
a02=V(3);
a03=V(4);


Bold=bold*(0:1:m-1)';   %sum k*bold_k  k=1...m-1

GradPhase=reshape(2*[Bold(1),0,Bold(2),0,Bold(3),0]'*ones(1,m-1),1,[]);    %gradient of the phase condition => first line of jacob
GradPhase=[0,Bold',GradPhase];  


Gradf01=[0,sigma,-sigma,zeros(1,6*m-5)];                                   %gradient of f01 =>2nd line of jabob

Gradf02=[0,-rho+a03,1,a01];                                                %gradient of f02 => 3rd line of jabob
Vcut=reshape(V(5:end),6,[]);

G2=2*[Vcut(5,:)',Vcut(6,:)',0*Vcut(1:2,:)',Vcut(1,:)',Vcut(2,:)'];
G2=reshape(G2',1,[]);
Gradf02=[Gradf02,G2];
 
  
Gradf03=[0,-a02,-a01,beta];                                                %gradient of f03 => 4th line of jabob
G3=-2*[Vcut(3,:)',Vcut(4,:)',Vcut(1,:)',Vcut(2,:)',0*Vcut(1:2,:)'];
Gradf03=[Gradf03,reshape(G3',1,[])];

%% grad F_k w.r.t. L , k>0 
V5=V(5:end);
Iodd=(1:2:6*(m-1)-1);
Ieven=(2:2:6*(m-1));
Kindex=reshape(repmat((1:1:m-1),[3,1]),[],1);
Grad_L(Iodd,1)=-V5(Ieven,1).*Kindex;
Grad_L(Ieven,1)=V5(Iodd,1).*Kindex;

%% grad F_k w.r.t a01, k>0 
Ga01=[0*Vcut(1:2,:)',Vcut(5,:)',Vcut(6,:)',-Vcut(3,:)',-Vcut(4,:)'];
Grad_a01=reshape(Ga01',1,[]);
 Grad_a01=Grad_a01';
 
%% grad F_k w.r.t. a02, k>0
Ga02=[0*Vcut(1:4,:)',-Vcut(1,:)',-Vcut(2,:)'];
Grad_a02=reshape(Ga02',1,[]);
 Grad_a02=Grad_a02';
 
%% grad F_k w.r.t. a03, k>0
Ga03=[0*Vcut(1:2,:)',Vcut(1,:)',Vcut(2,:)',0*Vcut(1:2,:)'];
Grad_a03=reshape(Ga03',1,[]);
 Grad_a03=Grad_a03';

%% grad F_k w.r.t a_l^j and b_l^j for k>0, l>0, j=1,2,3
 
 Mkj=zeros(6*(m-1));
 for k=1:m-1
     gradk=zeros(6,6*(m-1));
     for l=1:m-1
         gradk(:,6*(l-1)+1:6*l)=Jacobkl(k,l);
     end
     Mkj(6*(k-1)+1:6*k,:)=gradk;
 end
   
 jacob=[GradPhase;Gradf01;Gradf02;Gradf03;...
     [Grad_L,Grad_a01,Grad_a02,Grad_a03,Mkj]];
 
 
%% grad F_k w.r.t [a^1_l,b^1_l,a^2_l,b^2_l,a^3_l,b^3_l]   \in Mat(6x6), k,l>0,
 % if k==l the linear part need to be considered
 % in any case if l<=abs(k-(m-1)) the derivative is composed by 2 terms,
 % since in the convolution both c^i_l and c^_(-l) are present
 % NB a^i_(-l)=a^i_l, but b^i_(-l)=-b^i_l
 
    function  matr=Jacobkl(k,l)
        if k==l
            if l<=abs(k-(m-1))
                a2k1=V(4+6*(2*k-1)+1);
                b2k1=V(4+6*(2*k-1)+2);
                a2k2=V(4+6*(2*k-1)+3);
                b2k2=V(4+6*(2*k-1)+4);
                a2k3=V(4+6*(2*k-1)+5);
                b2k3=V(4+6*(2*k-1)+6);
                   
                matr=[sigma         -k*L           -sigma      0         0          0        ;...
                      k*L            sigma          0         -sigma     0          0        ;...
                     -rho+a03+a2k3   b2k3           1         -k*L       a01+a2k1   b2k1     ;...
                      b2k3          -rho+a03-a2k3   k*L        1         b2k1       a01-a2k1 ;...
                     -a02-a2k2      -b2k2          -a01-a2k1  -b2k1      beta      -L*k      ;...
                     -b2k2          -a02+a2k2      -b2k1      -a01+a2k1  L*k        beta]; 
            else
                matr=[sigma    -k*L     -sigma   0      0      0    ;...
                      k*L       sigma    0      -sigma  0      0    ;...
                     -rho+a03   0        1      -k*L    a01    0    ;...
                       0       -rho+a03  k*L     1      0      a01  ;...
                     -a02       0       -a01     0      beta  -L*k  ;...
                       0       -a02      0      -a01    L*k    beta]; 
            end
      
        else     %k not equal l
            
            j=abs(k-l);
            s=sign(k-l);
          
            a1=V(4+6*(j-1)+1);
            b1=V(4+6*(j-1)+2);
            a2=V(4+6*(j-1)+3);
            b2=V(4+6*(j-1)+4);
            a3=V(4+6*(j-1)+5);
            b3=V(4+6*(j-1)+6);
            
            if l<=abs(k-(m-1))
                ap1=V(4+6*(k+l-1)+1);
                bp1=V(4+6*(k+l-1)+2);
                ap2=V(4+6*(k+l-1)+3);
                bp2=V(4+6*(k+l-1)+4);
                ap3=V(4+6*(k+l-1)+5);
                bp3=V(4+6*(k+l-1)+6);
                matr=[0             0        0           0         0         0     ;...
                      0             0        0           0         0         0     ;...
                      a3+ap3     bp3-s*b3    0           0       a1+ap1   bp1-s*b1 ;...
                      bp3+s*b3  -ap3+a3      0           0      bp1+s*b1  -ap1+a1  ;...
                     -ap2-a2    -bp2+s*b2   -ap1-a1   -bp1+s*b1    0         0     ;...
                     -bp2-s*b2   ap2-a2     -bp1-s*b1  ap1-a1      0         0];
            else
                matr=[0       0      0     0    0      0     ;...
                      0       0      0     0    0      0     ;...
                      a3     -s*b3   0     0    a1     -s*b1 ;...
                      s*b3    a3     0     0    s*b1   a1    ;...
                      -a2     s*b2  -a1    s*b1  0     0     ;...
                      -s*b2  -a2    -s*b1  -a1   0     0];
            end
        end
    end

    
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function conv2=complex_convolution(A,B)
 
 % given A=(a0,a1,a2,....aN), B=(b0,b1,b2...bN)
 % first construct the conjugate estension (a_-N...a_-2,a_-1,a0,a1,...aN)
 % and (b_-N...b_-2,b_1,b0,b1,..bN) with a,b_(-i)=conj(a,b_i)
 % then compute conv(A,B)_k= sum_(j+l=k) aj*bl   k in [0,2*N] 
 
S=size(A);
if S(1)>1
    A=transpose(A);
    B=transpose(B);
end
  m=length(A);
  CA=conj(A(2:end));
  CB=conj(B(2:end));
  Atot=[fliplr(A),CA];
  Btot=[fliplr(CB),B];
  row=Atot;
  Matr=Atot;
  
  for i=1:m-1
      row=circshift(row,[0,1]);
      row(1)=0;
      Matr=[Matr;row];
  end
  Matr=[Matr;zeros(m-1,m),Matr(2:m,2:m)];
  conv2=Matr*Btot.';
  conv2=conv2.';
end