function [B,Pk]=newton_floquet_complex(Ak,B,Pk,mm)

%Newton scheme for the solution of the Floquet system, in unknowns B, P_k
%The scheme is independend on which system is considered.
%The information about the vector field are contained in the matrix Ak


% make the size of  Ak the same as Pk
if exist('mm','var')==0
    mA=(size(Ak,3)-1)/2;
    m=(size(Pk,3)-1)/2;
    if mA<m
        Ak=cat(3,zeros(size(Ak,1),size(Ak,2),m-mA),Ak,zeros(size(Ak,1),size(Ak,2),m-mA));
    else
        Ak(:,:,1:mA-m)=[];
        Ak(:,:,end+1-(mA-m):end)=[];
    end
else
    mA=(size(Ak,3)-1)/2;
    m=(size(Pk,3)-1)/2;
    if mA<mm
        Ak=cat(3,zeros(size(Ak,1),size(Ak,2),mm-mA),Ak,zeros(size(Ak,1),size(Ak,2),mm-mA));
    else
        Ak(:,:,1:mA-mm)=[];
        Ak(:,:,end+1-(mA-mm):end)=[];
    end
    if m<mm
        Pk=cat(3,zeros(size(Pk,1),size(Pk,2),mm-m),Pk,zeros(size(Pk,1),size(Pk,2),mm-m));
    else
        Pk(:,:,1:m-mm)=[];
        Pk(:,:,end+1-(m-mm):end)=[];
    end
    
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Pk=Pk+0.00001*(1+1i)*rand(size(Pk));
F=map_floquet_complex(B,Pk,Ak);

Ntest=0;
Freal=F_to_real(F);

display(['InfNorm=' num2str(norm(Freal,inf))]);
while norm(Freal,inf)>2*1e-13
    JJ=jacobian_floquet_complex(B,Pk,Ak);
    X=RQ_to_Xcomplex(B,Pk);
    
    X1=X-JJ\Freal;
    clear JJ
    X=X1;
    [B,Pk]=X_to_RQ_complex(X,size(B));
    
    F=map_floquet_complex(B,Pk,Ak);
    Freal=F_to_real(F);
    display(['InfNorm=' num2str(norm(Freal,inf))]);
    Ntest=Ntest+1;
    if Ntest>12
        if norm(Freal,inf)<9*1e-13
            break
        elseif norm(Freal,inf)>1
            disp('Newton scheme for the Floquet system does not converge');
            disp('Try to change  m');
          
            return
        else
            Ntest=Ntest-1;
        end
    end
    
end
[B,Pk]=X_to_RQ_complex(X,size(B));

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% Child Functions %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function F=map_floquet_complex(B,Pk,Ak)

global  T
m=(size(Pk,3)-1)/2;

phase=sum(Pk,3)-eye(size(Pk,1));

%P_0 corresponds to Pk(:,:,m+1)
F0=Pk(:,:,m+1)*B-conv(0,Ak,Pk);

F=cat(3,phase,F0);

cont=1;
for k=-m:m
    
    if k~=0
        Fk(:,:,cont)=2*1i*pi/(T)*k*Pk(:,:,m+k+1)-conv(k,Ak,Pk)+Pk(:,:,m+k+1)*B;
        cont=cont+1;
    end
end
F=cat(3,F,Fk);
end

function jac=jacobian_floquet_complex(B,Pk,Ak)

% Jacobian matrix for the Floquet system

global   T

m=(size(Pk,3)-1)/2;
sP1=size(Pk,1);
sP2=size(Pk,2);
sP=sP1*sP2;
sB1=size(B,1);
sB=sB1*sB1;


Bstar=kron(B.',eye(sB1));

jac_real_phase=[zeros(sP,sB),zeros(sP,sB),repmat([eye(sP),zeros(sP)],[1,2*m+1])];
jac_imag_phase=[zeros(sP,sB),zeros(sP,sB),repmat([zeros(sP),eye(sP)],[1,2*m+1])];

jac_real_F0_B=[kron(eye(sB1),real(Pk(:,:,m+1))),-kron(eye(sB1),imag(Pk(:,:,m+1)))];
jac_imag_F0_B=[kron(eye(sB1),imag(Pk(:,:,m+1))),kron(eye(sB1),real(Pk(:,:,m+1)))];


jac_real_F0=jac_real_F0_B;
jac_imag_F0=jac_imag_F0_B;

for k=-m:m
    D11=derConv(0,k,1,1,Ak);
    D12=derConv(0,k,1,2,Ak);
    D21=derConv(0,k,2,1,Ak);
    D22=derConv(0,k,2,2,Ak);
    
    jac_real_F0_Pk=[-kron(eye(sP2),D11),-kron(eye(sP2),D12)];
    jac_imag_F0_Pk=[-kron(eye(sP2),D21),-kron(eye(sP2),D22)];
    
    if k==0
        jac_real_F0_Pk=jac_real_F0_Pk+[real(Bstar),-imag(Bstar)];
        jac_imag_F0_Pk=jac_imag_F0_Pk+[imag(Bstar),real(Bstar)];
        
    end
    jac_real_F0=[jac_real_F0,jac_real_F0_Pk];
    jac_imag_F0=[jac_imag_F0,jac_imag_F0_Pk];
    
end
jac=sparse(zeros(2*(2*m+2)*sP));
jac(1:4*sP,:)=sparse([jac_real_phase;jac_imag_phase;jac_real_F0;jac_imag_F0]);
last_line=4*sP;
for j=-m:m
    
    if j~=0
        jac_real_Fj=zeros(sP,2*sP*(2*m+2));
        jac_imag_Fj=zeros(sP,2*sP*(2*m+2));
        jac_real_Fj(:,1:2*sP)=[kron(eye(sB1),real(Pk(:,:,m+j+1))),-kron(eye(sB1),imag(Pk(:,:,m+1+j)))];
        jac_imag_Fj(:,1:2*sP)=[kron(eye(sB1),imag(Pk(:,:,m+j+1))),kron(eye(sB1),real(Pk(:,:,m+1+j)))];
        last_col=2*sP;
        
        for k=-m:m
            D11=derConv(j,k,1,1,Ak);
            D12=derConv(j,k,1,2,Ak);
            D21=derConv(j,k,2,1,Ak);
            D22=derConv(j,k,2,2,Ak);
            
            jac_real_Fj_Pk=[-kron(eye(sP2),D11),-kron(eye(sP2),D12)];
            jac_imag_Fj_Pk=[-kron(eye(sP2),D21),-kron(eye(sP2),D22)];
            
            if k==j
                jac_real_Fj_Pk=jac_real_Fj_Pk+[real(Bstar),-imag(Bstar)-k*2*pi/T*eye(sB)];
                jac_imag_Fj_Pk=jac_imag_Fj_Pk+[imag(Bstar)+k*2*pi/T*eye(sB),real(Bstar)];
                
            end
            
            jac_real_Fj(:,last_col+1:last_col+2*sP)=jac_real_Fj_Pk;
            jac_imag_Fj(:,last_col+1:last_col+2*sP)=jac_imag_Fj_Pk;
            last_col=last_col+2*sP;
        end
        jac(last_line+1:last_line+2*sP,:)=sparse([jac_real_Fj;jac_imag_Fj]);
        
        last_line=last_line+2*sP;
    end
    
end
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function Conv=conv(k,Ak,Pk)

m=(size(Pk,3)-1)/2;
Conv=Ak(:,:,m+1)*Pk(:,:,m+k+1);
for k2=-m:m
    k1=k-k2;
    if k2~=k && abs(k1)<=m
        Conv=Conv+Ak(:,:,m+1+k1)*Pk(:,:,m+1+k2);
    end
end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

function  D=derConv(j,k,a1,a2,Ak)


m=(size(Ak,3)-1)/2;
k1=j-k;
if m+1+k1<1 || m+1+k1>2*m+1
    D=zeros(size(Ak,1),size(Ak,2));
    return
end
A=Ak(:,:,m+1+k1);

if a1==1 && a2==1
    D=real(A);
elseif a1==1 && a2==2
    D=-imag(A);
elseif a1==2 && a2==1
    D=imag(A);
elseif a1==2 && a2==2
    D=real(A);
end

end

function Freal=F_to_real(F)
m=size(F,3);
Freal=[];
for j=1:m
    Freal=[Freal;real(reshape(F(:,:,j),[],1));imag(reshape(F(:,:,j),[],1))];
end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function X=RQ_to_Xcomplex(R,Q)
clear X
m=size(Q,3);
X=[real(reshape(R,[],1));imag(reshape(R,[],1))];

for j=1:m
    X=[X;real(reshape(Q(:,:,j),[],1));imag(reshape(Q(:,:,j),[],1))];
end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [R,Q]=X_to_RQ_complex(X,dim)
clear R Q
s1=dim(1);
s2=dim(2);
ss=s1*s2;
R=reshape(X(1:ss),s1,s2)+1i*reshape(X(ss+1:2*ss),s1,s2);
Xc=X(2*ss+1:end);

m=size(Xc,1)/(2*ss);

for j=1:m
    Q(:,:,j)=reshape(Xc(2*ss*(j-1)+1:2*ss*(j-1)+ss),s1,s2)+1i*reshape(Xc(2*ss*(j-1)+ss+1:2*ss*(j-1)+2*ss),s1,s2);
end
end



